#include <Application.h>
#include <String.h>

#include <MediaRoster.h>
#include <MediaNode.h>
#include <ParameterWeb.h>

#include <stdio.h>
#include <string.h>

BMediaRoster *gMediaRoster;

void processGroup(BParameterGroup *group, int32 identation);

int main(void) {
	media_node	outNode;
	status_t	err;

	printf("** Media Test v1.0 **\n\n");
	printf("This application will generate a report  about all controls\n");
	printf("reported by the media add-on being used by your audio card.\n");
	printf("Please send the output to bga@bug-br.org.br and include the\n");
	printf("following information:\n\n");
	printf("- Sound Card Model/Maker/Chipset\n");
	printf("- Driver being used (if not the default that comes with R5)\n");
	printf("- BeOS version (Zeta/BeOS R5/BeOS R5 + MKB1/Dano)\n\n");	 
	printf("This version iterates through the BParameterGroups and generates\n");
	printf("a tree representation showing all the controls found.\n\n");
	BApplication myApp("application/x-vnd.BGA-MediaTest");
	
	gMediaRoster = BMediaRoster::Roster(&err);
	
	if (!gMediaRoster || err != B_OK) {
		printf("Can't get the Media Roster. Is the Media Server running?\n"); 
	} else {
		if (gMediaRoster->GetAudioOutput(&outNode) == B_OK) {
			BParameterWeb * web;
			if (gMediaRoster->GetParameterWebFor(outNode, &web) == B_OK && web) {
				int32 numGroups     = web->CountGroups();
				int32 numParameters = web->CountParameters();
				
				printf("Found the ParameterWeb for the audio output node. %d groups. %d parameters.\n", numGroups, numParameters);

				for (int32 i = 0; i < numGroups; i++) {
					processGroup(web->GroupAt(i), 4);
				}
			}
		}
	}
}

void processGroup(BParameterGroup *group, int32 identation) {
	int32 numGroups     = group->CountGroups();
	int32 numParameters = group->CountParameters();

	for (int i = 0; i < identation; i++)
		printf(" ");
			
	printf("Found group \"%s\". %d subgroups. %d parameters.\n", group->Name(), numGroups, numParameters);		
	
	for (int32 i = 0; i < numParameters; i++) {
		BParameter *parameter = group->ParameterAt(i);

		for (int i = 0; i < identation + 4; i++)
			printf(" ");
		
		printf("Found parameter \"%s\" (%s).", parameter->Name(), parameter->Kind());
	}

	for (int32 i = 0; i < numGroups; i++)
		processGroup(group->GroupAt(i), identation + 4);
}
